<?php
/* $Id: seo.url.php 107 2007-03-02 16:17:41Z dreamscape $ */
/**
 * @package SeoUrl
 * @subpackage Core
 * @name seo.core.url
 */

if (!defined('IS_ADMIN_FLAG')) {
	die('Illegal Access');
}


define('SEO_URL_BASE_DIR', dirname(__FILE__));

require_once(SEO_URL_BASE_DIR . '/seo.installer.php');
require_once(SEO_URL_BASE_DIR . '/seo.settings.php');
require_once(SEO_URL_BASE_DIR . '/seo.plugin_loader.php');
require_once(SEO_URL_BASE_DIR . '/seo.url_plugins.php');
require_once(SEO_URL_BASE_DIR . '/seo.char_plugins.php');
require_once(SEO_URL_BASE_DIR . '/seo.compatibility/http_build_query.php');



/**
 * Seo Url Transformations for Zen Cart
 *
 * @version $Rev: 107 $
 * @author Josh Dechant <josh@flexiecommerce.com>
 * @copyright Copyright &copy; 2006 - 2007 JD Productions, LLC. All Rights Reserved.
 * @license http://www.opensource.org/licenses/gpl-license.php GNU General Public License
 * @package SeoUrl
 * @subpackage Core
 */
class SeoUrl {


	/**
	 * @var object Object Database access object
	 * @access protected
	 **/
	var $db;


	/**
	 * @var object SeoSettings
	 * @access protected
	 */
	var $settings;


	/**
	 * @var object SeoPlugins
	 * @access protected
	 */
	var $plugins;


	/**
	 * @var integer Current session's language ID
	 * @access protected
	 */
	var $languageId;


	/**
	 * Class Constructor
	 *
	 * @param object Object $db Database access object
	 * @return void
	 */
	function SeoUrl(&$db) {
		$this->db =& $db;

		$this->settings =& new SeoSettings($db);

		$installer =& new SeoInstaller($db, $this->settings);
		$installer->checkInstallation();


		if ($this->settings->isEnabled() === false) {
			return;
		}


		$this->plugins =& new SeoUrlPlugins(
			new SeoPluginLoader('url'),
			$this->settings->getPlugins('url'),
			new SeoCharPlugins(
				new SeoPluginLoader('char'),
				$this->settings->getPlugins('char')
			),
			$this->settings->getDefaultExtension()
		);
	}



	function setupLanguage($id) {
		$this->languageId = (int)$id;

		if ($this->settings->isEnabled() === false) {
			return;
		}

		$this->plugins->setupCache(new SeoPluginCache($this->db, $this->languageId));
	}




	/**
	 * Match an SEO Url
	 *
	 * @return void
	 * @access protected
	 */
	function matchSeoUrl() {
		if ($this->settings->isEnabled() === false) {
			return;
		}

		// allow passing url in GET param for servers without correct PATH_INFO
		if (!isset($_SERVER['PATH_INFO']) || empty($_SERVER['PATH_INFO'])) {
			if (isset($_GET['seo_url']) && !empty($_GET['seo_url'])) {
				$_SERVER['PATH_INFO'] = $_GET['seo_url'];
				unset($_GET['seo_url']);
			} else {
				return;
			}
		}

		$seo_url = ltrim($_SERVER['PATH_INFO'], '/');

		if (empty($seo_url)) {
			return;
		}


		$_GET['main_page'] = false;

		if ($matches = $this->plugins->matchSeoUrl($seo_url)) {
			$_GET['main_page'] = $matches[0];

			if (isset($matches[1]) && (sizeof($matches[1]) > 0)) {
				foreach ($matches[1] as $key => $value) {
					$_GET[$key] = $value;
				}
			}
		}

		// Put junk into $_GET['main_page'] so Zen Cart will throw a 404 header
		if ($_GET['main_page'] === false) {
			$_GET['main_page'] = '1-qwerty-lane';
		}
	}


	/**
	 * ...
	 */
	function buildHrefLink($page = '', $parameters = '', $connection = 'NONSSL', $add_session_id = true, $static = false, $use_dir_ws_catalog = true) {
		/**
		 * don't rewrite when disabled
		 */
		if ($this->settings->isEnabled() === false) {
			return $this->buildStockHrefLink($page, $parameters, $connection, $add_session_id, true,
				$static, $use_dir_ws_catalog);
		}


		/**
		 * Fix up $page index.php?main_page=index&cPath=21
		 */
		$page = preg_replace('/(index\.php\??)?main_page=/', '', $page);


		/**
		 * don't rewrite pages we should skip
		 */
		if ($this->plugins->skipSeoUrl($page, $parameters, $connection) === true) {
			return $this->buildStockHrefLink($page, $parameters, $connection, $add_session_id, true, $static, $use_dir_ws_catalog);
		}

		/**
		 * Temporarily clean up "&amp;" in $page
		 */
		$page = str_replace('&amp;', '&', $page);


		/**
		 * Move any params in $page to $parameters
		 */
		$matches = split('[?&]', $page, 2);
		$page = $matches[0];

		if (isset($matches[1])) {
			if (empty($parameters)) {
				$parameters = $matches[1];
			} else {
				$parameters = $matches[1] . '&' . $parameters;
			}
		}


		/**
		 * Set default page
		 */
		if (empty($page)) {
			$page = 'index';
		}


		/**
		 * Temporarily clean up "&amp;" in $parameters
		 */
		$parameters = str_replace('&amp;', '&', $parameters);

		/**
		 * Get base link
		 */
		$link = $this->getBaseLink($page, $connection, $use_dir_ws_catalog);

		$separator = '?';

		$link .= ltrim($this->parseParams($page, $parameters, $separator), '/');

		$link .= $this->addSid($add_session_id, $connection, $separator);

		$link = str_replace('&', '&amp;', $link);

		return $link;
	}


	/**
	 * ...
	 */
	function buildStockHrefLink($page = '', $parameters = '', $connection = 'NONSSL', $add_session_id = true, $search_engine_safe = true, $static = false, $use_dir_ws_catalog = true) {
		return zen_href_link_stock($page, $parameters, $connection, $add_session_id, true, $static, $use_dir_ws_catalog);
	}


	/**
	 * Get URL base
	 */
	function getBaseLink($page, $connection, $use_dir_ws_catalog) {
		if (!in_array($connection, array('NONSSL', 'SSL'))) {
			trigger_error('Unable to determine connection method on the page link', E_USER_ERROR);
		}

		$link = HTTP_SERVER;
		if (($connection == 'SSL') && (ENABLE_SSL == 'true')) {
			$link = HTTPS_SERVER;
		}

		if ($use_dir_ws_catalog == true) {
			if (($connection == 'SSL') && (ENABLE_SSL == 'true') && defined('DIR_WS_HTTPS_CATALOG')) {
		        $link .= DIR_WS_HTTPS_CATALOG;
			} else {
		        $link .= DIR_WS_CATALOG;
			}
		}

		return $link;
	}


	/**
	 * ...
	 */
	function parseParams($page, $param_string, &$separator) {
		$url = false;
		$params = array();

		if (!empty($param_string)) {
			parse_str($param_string, $params);
			if ($tmp_url = $this->plugins->generateSeoUrl($page, $params)) {
				$url = $tmp_url['url'];
			}
		}


		if ($url === false) {
			$url = '';
			if (($tmp_url = $this->plugins->generateSeoUrl($page))) {
				$url = $tmp_url['url'];
			}
		}


		if ($url !== '') {
			$url = $this->settings->getPageBase() . '/' . $url;
		}


		$exclude = array('seo_url');

		if (isset($tmp_url['exclude']) && ($tmp_url['exclude'] !== false)) {
			if (!is_array($tmp_url['exclude'])) {
				$tmp_url['exclude'] = array($tmp_url['exclude']);
			}
			$exclude = array_merge($exclude, $tmp_url['exclude']);
		}


		foreach ($exclude as $param) {
			unset($params[$param]);
		}


		if (sizeof($params) > 0) {
			$url .= $separator . http_build_query($params);
			$separator = '&';
		}

		return $url;
	}


	/**
	 * Add the session ID when moving from different HTTP and HTTPS servers, or when SID is defined
	 */
	function addSid($add_session_id, $connection, $separator = '&') {
		if (SESSION_FORCE_COOKIE_USE == 'True') {
			return '';
		}

		if (($add_session_id == true) && ($GLOBALS['session_started'] == true)) {
			if (defined('SID') && (SID != '')) {
				return $separator . SID;
			} elseif (($GLOBALS['request_type'] != $connection) && (ENABLE_SSL == 'true')) {
				if ($GLOBALS['http_domain'] != $GLOBALS['https_domain']) {
					return $separator . session_name() . '=' . session_id();
				}
			}
		}

		return '';
	}


	/**
	 * Check if a URL needs to be redirected to its SEO URL (sending 301 header),
	 * and perform redirection if necessary
	 * @return void
	 * @access protected
	 */
	function checkRedirection() {
		if ($this->settings->isEnabled() === false) {
			return;
		}

		if ($this->settings->is301Enabled() === false) {
			return;
		}

		if ($GLOBALS['request_type'] === 'SSL') {
			return;
		}

		$uri = basename($_SERVER['REQUEST_URI']);

		if ($this->isSeoUri($uri) === false) {
			$parsedUri = parse_url($uri);

			/**
			 * zen cart attributes use illegal url characters
			 */
			if ($this->isAttributeUri($uri) && !preg_match('/https?/', $parsedUri['scheme'])) {
				$parsedUriTmp = $parsedUri;
				$parsedUri = parse_url($parsedUriTmp['scheme']);
				$parsedUri['query'] = preg_replace('/products_id=([0-9]+)/', 
					'products_id=$1:' . $parsedUriTmp['path'], $parsedUri['query']);
			}

			$this->doRedirection($parsedUri['query']);
		}
	}

	/**
	 * Redirect a URL to its SEO URL
	 * @param string $query A URL's query string
	 * @return void
	 * @access protected
	 */
	function doRedirection($query) {
		parse_str($query, $params);

		if (!isset($params['main_page'])) {
			return;
		}

		$page = $params['main_page'];
		unset($params['main_page']);

		$params = http_build_query($params);

		$url = $this->buildHrefLink($page, $params, 'NONSSL', false);

		/**
		 * cleanup url for redirection
		 */
		$url = str_replace('&amp;', '&', $url);

		header('HTTP/1.1 301 Moved Permanently');
		header('Location: ' . $url);
		session_write_close();
		exit;
	}

	/**
	 * Basic check if a URL is an SEO URL
	 * @param string $uri A valid URI
	 * @return boolean
	 * @access protected
	 */
	function isSeoUri($uri) {
		if (!preg_match('/main_page=/', $uri)) {
			return true;
		}

		return false;
	}

	/**
	 * Basic check if a URL has product attributes
	 * @param string $uri A valid URI
	 * @return boolean
	 * @access protected
	 */
	function isAttributeUri($uri) {
		if (preg_match('/products_id=([0-9]+):([a-zA-z0-9]{32})/', $uri)) {
			return true;
		}

		return false;
	}

}
